package ch.epfl.collections;

import java.util.ArrayList;
import java.util.Collections;

import org.junit.Test;
import static org.junit.Assert.*;

abstract public class TestSet {
    abstract Set<Integer> newEmptyIntSet();

    @Test
    public void testIsEmpty() {
        Set<Integer> s = newEmptyIntSet();
        assertTrue(s.isEmpty());
        s.add(1);
        assertFalse(s.isEmpty());
        s.remove(1);
        assertTrue(s.isEmpty());
    }

    @Test
    public void testSize() {
        Set<Integer> s = newEmptyIntSet();
        for (int i = 0; i < 20; ++i) {
            assertEquals(i, s.size());
            s.add(i);
        }
    }
    
    @Test
    public void testAdd() {
    	Set<Integer> s = newEmptyIntSet();
    	assertTrue(s.isEmpty());
    	s.add(1);
    	assertTrue(s.contains(1));
    	s.add(1);
    	assertTrue(s.size() == 1);
    	s.remove(1);
    	assertTrue(s.isEmpty());
    	
    	int[] elems = new int[] { 3, 1, 5, 7, 9, 15, 13, 11, 4, 6, 8, 10, 12, 14, 16, 17, 18, 19, 20 };
    	for (int e : elems)
    		s.add(e);
    	assertTrue(s.size() == elems.length);
    }

    @Test
    public void testRemove() {
        Set<Integer> s = newEmptyIntSet();
        int[] elems = new int[] { 3, 1, 5, 7, 9, 15, 13, 11 };
        for (int e: elems)
            s.add(e);
        for (int e: elems) {
            assertTrue(s.contains(e));
            s.remove(e);
            assertFalse(s.contains(e));
        }
        assertTrue(s.isEmpty());
    }

    @Test
    public void testHeavyLoad() {
        ArrayList<Integer> elems = new ArrayList<Integer>();
        for (int i = 0; i < 10000; ++i)
            elems.add(i);
        Collections.shuffle(elems);
        
        Set<Integer> s = newEmptyIntSet();
        for (int elem: elems) {
            s.add(elem);
            assertTrue(s.contains(elem));
        }
        for (int i = 20000; i < 30000; ++i) {
            s.remove(i);
        }
        for (int elem: elems) {
            s.remove(elem);
            assertFalse(s.contains(elem));
        }
        assertEquals(0, s.size());
    }
    
    @Test
    public void testContains() {
        Set<Integer> s = newEmptyIntSet();
        assertFalse(s.contains(1));
        s.add(1);
        assertTrue(s.contains(1));
        s.add(0);
        s.add(2);
        assertFalse(s.contains(-1));
        assertTrue(s.contains(0));
        assertTrue(s.contains(1));
        assertTrue(s.contains(2));
        assertFalse(s.contains(3));
    }

    @Test
    public void testIterator() {
        Set<Integer> s = newEmptyIntSet();
        ArrayList<Integer> l = new ArrayList<Integer>();
        for (int i = 0; i < 32; ++i)
            l.add(i);
        Collections.shuffle(l);
        for (int i: l)
            s.add(i);
        int bitSet = 0;
        for (int i: s) {
            assertTrue(0 <= i && i < 32);
            final int mask = 1 << i;
            assertTrue((bitSet & mask) == 0);
            bitSet |= mask;
        }
        assertEquals(0xFFFFFFFF, bitSet);
    }
}
